/*
 * Main function
 *
 * (c) Aki Tossavainen 2004

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
#include "eics.h"

int
main (int argc, const char **argv)
{
  int state;
  /* Run mode */
  enum
  {
    eics_init,
    eics_check,
    eics_update,
    eics_test
  } runmode;
  /* Print our information */
  printf
    ("Easy Integrity check system v%s Copyright (c) 2004 Aki Tossavainen\n",
     PACKAGE_VERSION);
  printf
    ("This software comes with ABSOLUTELY NO WARRANTY; see WARRANTY for details\n");
  printf
    ("This is free software and you are welcome to redistribute it under\ncertain conditions; see LICENSE for details\n\n");
  /* Initialize stuff */
  initHash ();
  initPolicy ();
  ERR_load_crypto_strings ();
  OpenSSL_add_all_algorithms ();

  /* Check command line arguments */
  if (argc != 2)
    {
      fprintf (stderr, "Usage: %s [-icut]\n", argv[0]);
      return 1;
    }

  if (!strcmp (argv[1], "-i"))
    {
      runmode = eics_init;
    }
  else if (!strcmp (argv[1], "-c"))
    {
      runmode = eics_check;
    }
  else if (!strcmp (argv[1], "-u"))
    {
      runmode = eics_update;
    }
  else if (!strcmp (argv[1], "-t"))
    {
      runmode = eics_test;
    }
  else
    {
      fprintf (stderr, "Usage: %s [-icut]\n", argv[0]);
      return 1;
    }
  /* Check that we can access files */
  if ((access (EICS_PUB_KEY, R_OK)) || (access (EICS_PRIV_KEY, R_OK)))
    {
      if (runmode == eics_init)
	{
	  generateNewKeys (EICS_PUB_KEY, EICS_PRIV_KEY);
	}
      else
	{
	  fprintf (stderr,
		   "Keypair not found, please run with -i parameter to initialize\n");
	}
    }

  if (access (EICS_DB_FILE, R_OK))
    {
      if (runmode != eics_init)
	{
	  fprintf (stderr,
		   "Database file not readable, please run with -i parameter to initialize\n");
	  return 1;
	}
    }

  if ((runmode == eics_update) && (access (EICS_DB_FILE, R_OK | W_OK)))
    {
      fprintf (stderr,
	       "Database file not readwritable, please check your permissions on the file\n");
      return 1;
    }

  /* Load policy file */
  if (loadPolicy
      (EICS_CONFIG_FILE, EICS_CONFIG_FILE_SIG,
       (runmode == eics_check ? 0 : 1)))
    {
      fprintf (stderr, "Error loading policy file\n");
      return -1;
    }

  /* Load database, if not initializing */
  if (runmode != eics_init)
    {
      if (loadDatabase (EICS_DB_FILE, EICS_DB_FILE_SIG))
	{
	  fprintf (stderr, "Unable to load database\n");
	  return -1;
	}
    }
  /* So that not every piece of resource is taken */
#ifdef HAVE_NICE
  if (runmode == eics_check)
    {
      nice (20);
    }
  else
    {
      nice (0);
    }
#endif

  /* Start processing */
  printf ("Starting filesystem check\n");
  /* Check for new or modified files */
  state = processDirectory (getFirstPolicyDir (), getFirstPolicyFlags ());
  /* Check for files gone missing */
  state += pruneDatabase ();

  /* If there has been changes */
  if (state)
    {
      /* and we are updating or creating */
      if (runmode != eics_check)
	{
	  /* then we really should at least try to write it into a file */
	  if (writeDatabase
	      (EICS_DB_FILE, EICS_DB_FILE_TMP, EICS_DB_FILE_SIG))
	    {
	      fprintf (stderr, "Unable to write database\n");
	      return -1;
	    }
	}
    }
  return 0;
}
