/*
 * This one creates a ripemd and sha1 meshes.
 *
 * (c) Aki Tossavainen 2004

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include "eics.h"

/* Create the meshes and store them into mesh */
int
meshFile (const char *file, struct mesh_s *mesh)
{
  FILE *in;
  char readbuf[1024];
  unsigned int rlen;
  EVP_MD_CTX ctx_sha;
  EVP_MD_CTX ctx_ripemd;

  if (!mesh)
    return -1;

  /* Try to open the file, just for reading */
  if (!(in = fopen (file, "rb")))
    return -1;

  /* Init digests */
  EVP_DigestInit (&ctx_sha, EVP_sha1 ());
  EVP_DigestInit (&ctx_ripemd, EVP_ripemd160 ());

  /* As long as we have stuff */
  while ((rlen = fread (readbuf, 1, sizeof (readbuf), in)) > 0)
    {
      EVP_DigestUpdate (&ctx_sha, readbuf, rlen);
      EVP_DigestUpdate (&ctx_ripemd, readbuf, rlen);
    }
  /* Check that no error occured */
  if (ferror (in))
    {
      EVP_MD_CTX_cleanup (&ctx_sha);
      EVP_MD_CTX_cleanup (&ctx_ripemd);
      fclose (in);
      return -1;
    }

  fclose (in);

  /* Store the digests */
  EVP_DigestFinal (&ctx_sha, mesh->sha1, &rlen);
  EVP_DigestFinal (&ctx_ripemd, mesh->ripemd, &rlen);

  return 0;
}
